@@ -9,7 +9,7 @@
 
 package Math::Formula;
 use vars '$VERSION';
-$VERSION = '0.12';
+$VERSION = '0.13';
 
 
 use warnings;
@@ -29,13 +29,24 @@ sub new(%)
 {	my ($class, $name, $expr, %self) = @_;
 	$self{_name} = $name;
 	$self{_expr} = $expr;
-	(bless {}, $class)->init(\%self) }
+	(bless {}, $class)->init(\%self);
+}
 
 sub init($)
 {	my ($self, $args) = @_;
-	$self->{MSBE_name}    = $args->{_name} or panic "every formular requires a name";
-	$self->{MSBE_expr}    = $args->{_expr} or panic "every formular requires an expression";
-	$self->{MSBE_returns} = $args->{returns};
+	my $name = $self->{MSBE_name} = $args->{_name} or panic "every formular requires a name";
+
+	my $expr    = $args->{_expr} or panic "every formular requires an expression";
+	my $returns = $self->{MSBE_returns} = $args->{returns};
+
+	if(ref $expr eq 'SCALAR')
+	{	$expr = MF::STRING->new(undef, $$expr);
+	}
+	elsif(! ref $expr && $returns && $returns->isa('MF::STRING'))
+	{	$expr = MF::STRING->new(undef, $expr);
+	}
+
+	$self->{MSBE_expr} = $expr;
 	$self;
 }
 
@@ -63,7 +74,7 @@ sub _test($$)
 ### PARSER
 ###
 
-my $multipliers = MF::INTEGER->_multipliers;
+my $match_int   = MF::INTEGER->_match;
 my $match_float = MF::FLOAT->_match;
 my $match_name  = MF::NAME->_match;
 my $match_date  = MF::DATE->_match;
@@ -88,7 +99,7 @@ sub _tokenize($)
 	$s =~ m/ ^
 	(?: \s*
 	  (?| \# (?: \s [^\n\r]+ | $ ) \
-		| ( true | false )	(?{ push @t, MF::BOOLEAN->new($+) })
+		| ( true\b | false\b )	(?{ push @t, MF::BOOLEAN->new($+) })
 		| ( \" (?: \\\" | [^"] )* \" )
 							(?{ push @t, MF::STRING->new($+) })
 		| ( \' (?: \\\' | [^'] )* \' )
@@ -100,8 +111,7 @@ sub _tokenize($)
 		| ( $match_date )	(?{ push @t, MF::DATE->new($+) })
 		| ( $match_time )	(?{ push @t, MF::TIME->new($+) })
 		| ( $match_float )	(?{ push @t, MF::FLOAT->new($+) })
-		| ( [0-9][0-9_]*(?:$multipliers)? )
-							(?{ push @t, MF::INTEGER->new($+) })
+		| ( $match_int )	(?{ push @t, MF::INTEGER->new($+) })
 		| \(				(?{ push @t, MF::PARENS->new('(', ++$parens_open) })
 		| \)				(?{ push @t, MF::PARENS->new(')', $parens_open--) })
 		| $
@@ -229,7 +239,8 @@ sub toType($)
 	my $match = sub { my $type = shift; my $match = $type->_match; qr/^$match$/ };
 
 	return 
-	    $data =~ /^[+-]?[0-9]+$/         ? MF::INTEGER->new(undef, $data)
+		ref $data eq 'SCALAR'            ? MF::STRING->new($data)
+	  : $data =~ /^[+-]?[0-9]+$/         ? MF::INTEGER->new(undef, $data)
 	  : $data =~ /^[+-]?[0-9]+\./        ? MF::FLOAT->new(undef, $data)
 	  : $data =~ /^(?:true|false)$/      ? MF::BOOLEAN->new($data)
 	  : ref $data eq 'Regexp'            ? MF::REGEXP->new(undef, $data)
@@ -238,7 +249,7 @@ sub toType($)
 	  : $data =~ $_match{'MF::DATE'}     ? MF::DATE->new($data)
 	  : $data =~ $_match{'MF::DURATION'} ? MF::DURATION->new($data)
 	  : $data =~ /^(['"]).*\1$/          ? MF::STRING->new($data)
-	  :                                    MF::STRING->new(undef, $data);
+	  : error __x"not an expression (string needs \\ ) for '{data}'", data => $data;
 }
 
 #--------------------------
