@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 2.03.
 package Math::Formula::Config;
 use vars '$VERSION';
-$VERSION = '0.12';
+$VERSION = '0.13';
 
 
 use warnings;
@@ -37,4 +37,11 @@ sub path_for($$)
 	File::Spec->catfile($self->directory, $file);
 }
 
+#----------------------
+
+sub save($%) { die "Save under construction" }
+
+
+sub load($%) { die "Load under construction" }
+
 1;
