@@ -5,7 +5,7 @@
 
 package Math::Formula::Context;
 use vars '$VERSION';
-$VERSION = '0.12';
+$VERSION = '0.13';
 
 
 use warnings;
@@ -22,22 +22,27 @@ sub _default($$$$)
 	my $form
 	  = ! $value         ? $type->new(undef, $default)
 	  : ! blessed $value ? ($value ? Math::Formula->new($name, $value) : undef)
-	  : $value->isa('Math::Formula') ? $value
+	  : $value->isa('Math::Formula')       ? $value
+	  : $value->isa('Math::Formula::Type') ? $value
 	  : error __x"unexpected value for '{name}' in #{context}", name => $name, context => $self->name;
 }
 
 sub init($)
 {	my ($self, $args) = @_;
-	my $name   = $self->{MFC_name}   = $args->{name} or error __x"context without a name";
+	my $name   = $args->{name} or error __x"context requires a name";
+	my $node   = blessed $name ? $name : MF::STRING->new(undef, $name);
+	$self->{MFC_name}   = $node->value;
 
+	my $now;
 	$self->{MFC_attrs} = {
-		ctx_name       => MF::STRING->new($name),
-		ctx_version    => $args->{version}    ? MF::STRING->new($args->{version}) : undef,
-		ctx_created    => $self->_default(created => 'MF::DATETIME', $args->{created}, DateTime->now),
-		ctx_updated    => $args->{updated} ? MF::DATETIME->new(updated => $args->{updated}) : undef,
-		ctx_mf_version => MF::STRING->new(undef, $args->{mf_version} // $Math::Formula::VERSION),
+		ctx_name       => $node,
+		ctx_version    => $self->_default(version => 'MF::STRING',   $args->{version}, "1.00"),
+		ctx_created    => $self->_default(created => 'MF::DATETIME', $args->{created}, $now = DateTime->now),
+		ctx_updated    => $self->_default(updated => 'MF::DATETIME', $args->{updated}, $now //= DateTime->now),
+		ctx_mf_version => $self->_default(mf_version => 'MF::STRING', $args->{mf_version}, $Math::Formula::VERSION),
 	};
 
+	$self->{MFC_lead}   = $args->{lead_expressions} // '';
 	$self->{MFC_forms}  = { };
 	$self->{MFC_frags}  = { };
 	if(my $forms = $args->{formulas})
@@ -59,7 +64,8 @@ sub _index()
 
 #--------------
 
-sub name   { $_[0]->{MFC_name} }
+sub name            { $_[0]->{MFC_name} }
+sub lead_expressions { $_[0]->{MFC_lead} }
 
 #--------------
 
@@ -99,22 +105,39 @@ sub add(@)
 
 
 sub addFormula(@)
-{	my ($self, $first) = (shift, shift);
-	my $next = $_[0];
+{	my ($self, $name) = (shift, shift);
+	my $next  = $_[0];
+	my $forms = $self->{MFC_forms};
 
-	my ($name, $form) =
-	  !@_ && blessed $first && $first->isa('Math::Formula')
-	? ($first->name, $first)
-	: @_==1 && !ref $first && blessed $next && $next->isa('Math::Formula')
-	? ($first, $next)
-	: @_==1 && !ref $first && ref $next eq 'ARRAY'
-	? ($first, Math::Formula->new($first, @{$next}))
-	: @_ && !ref $first &&
-		 (!ref $next || ref $next eq 'CODE' || blessed $next && $next->isa('Math::Formula::Type'))
-	? ($first, Math::Formula->new($first, @_))
-	: panic __x"formula declaration '{name}' not understood", name => $first;
+	if(ref $name)
+	{	return $forms->{$name->name} = $name
+			if !@_ && blessed $name && $name->isa('Math::Formula');
+	}
+	elsif(! ref $name && @_)
+	{	return $forms->{$name} = $next
+			if @_==1 && blessed $next && $next->isa('Math::Formula');
 
-	$self->{MFC_forms}{$name} = $form;
+		return $forms->{$name} = Math::Formula->new($name, @_)
+			if ref $next eq 'CODE';
+
+		return $forms->{$name} = Math::Formula->new($name, @_)
+			if blessed $next && $next->isa('Math::Formula::Type');
+
+		my ($data, %attrs) = @_==1 && ref $next eq 'ARRAY' ? @$next : $next;
+		if(my $r = $attrs{returns})
+		{	my $typed = $r->isa('MF::STRING') ? $r->new(undef, $data) : $data;
+			return $forms->{$name} = Math::Formula->new($name, $typed, %attrs);
+		}
+
+		if(length(my $leader = $self->lead_expressions))
+		{	my $typed  = $data =~ s/^\Q$leader// ? $data : \$data;
+			return $forms->{$name} = Math::Formula->new($name, $typed, %attrs);
+		}
+
+		return $forms->{$name} = Math::Formula->new($name, $data, %attrs);
+	}
+
+	error __x"formula declaration '{name}' not understood", name => $name;
 }
 
 
