@@ -8,7 +8,7 @@ use v5.16;  # fc
 
 package Math::Formula::Type;
 use vars '$VERSION';
-$VERSION = '0.12';
+$VERSION = '0.13';
 
 use base 'Math::Formula::Token';
 
@@ -85,6 +85,14 @@ package
 
 use base 'Math::Formula::Type';
 
+# $class->new($token, $value, %options)
+# When the value is derived from an expression, this should result in 1 or 0
+sub new($$@)
+{	my ($class, $token, $value) = (shift, shift, shift);
+	defined $token or $value = $value ? 1 : 0;
+	$class->SUPER::new($token, $value, @_);
+}
+
 sub prefix($)
 {	my ($self, $op, $context) = @_;
 	if($op eq 'not')
@@ -122,17 +130,16 @@ use base 'Math::Formula::Type';
 use Unicode::Collate ();
 my $collate = Unicode::Collate->new;  #XXX which options do we need?
 
-# Perl's false is 'undef': convert it to '0'
 sub new($$@)
-{	my ($class, $token, $value) = (shift, shift, shift);
-	$value //=0 unless defined $token;
-	$class->SUPER::new($token, $value, @_);
+{   my ($class, $token, $value) = (shift, shift, shift);
+    ($token, $value) = (undef, $$token) if ref $token eq 'SCALAR';
+    $class->SUPER::new($token, $value, @_);
 }
 
 sub _token($) { '"' . ($_[1] =~ s/[\"]/\\$1/gr) . '"' }
 
 sub _value($)
-{	my $token = $_[1];
+{	my $token = $_[1] // '';
 
 	  substr($token, 0, 1) eq '"' ? $token =~ s/^"//r =~ s/"$//r =~ s/\\([\\"])/$1/gr
 	: substr($token, 0, 1) eq "'" ? $token =~ s/^'//r =~ s/'$//r =~ s/\\([\\'])/$1/gr
@@ -237,7 +244,7 @@ sub infix($$$)
 my $gibi        = 1024 * 1024 * 1024;
 
 my $multipliers = '[kMGTEZ](?:ibi)?\b';
-sub _multipliers { $multipliers }
+sub _match { "[0-9][0-9_]* (?:$multipliers)?" }
 
 my %multipliers = (
 	k => 1000, M => 1000_000, G => 1000_000_000, T => 1000_000_000_000, E => 1e15, Z => 1e18,
